package es.uca.xmleye.gui;

/*
 Copyright (C) 2008 Antonio García Domínguez 

 Integra funciones de ejemplo de InfoNode Tabbed Panel 
 (GPL para uso no comercial)

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.ResourceBundle;
import java.util.Locale;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import net.infonode.tabbedpanel.*;
import net.infonode.tabbedpanel.theme.*;
import net.infonode.tabbedpanel.titledtab.*;

import es.uca.xmleye.ExcepcionAnalisis;
import es.uca.xmleye.presentacion.ModPresentacionDocumento;
import es.uca.xmleye.presentacion.ModPresentacionVentanaPrincipal;
import es.uca.xmleye.xslt.ExcepcionTransformacion;

class ListaDocumentos extends TabbedPanel implements TabListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 3010400667110209512L;

	// Recursos para iconos e internacionalización
	private static final ResourceBundle res;

	// Modelo de presentación de la ventana principal
	private final ModPresentacionVentanaPrincipal modVentanaPrincipal;
	
	// Modelo vacío de documento (para cuando no hay nada abierto)
	private final ModPresentacionDocumento docVacio;

	// Iconos para botones de cierre
	private static ImageIcon iconoSel;
	private static ImageIcon iconoDesel;

	static {
		res = ResourceBundle.getBundle(
				"es.uca.xmleye.gui.l10n.ResVentanaPrincipal", Locale
						.getDefault());
		iconoSel = new ImageIcon(res.getString("icon_close_selected"));
		iconoDesel = new ImageIcon(res.getString("icon_close_unselected"));
	}

	private TitledTabProperties titledTabProperties;
	private TabbedPanelTitledTabTheme tema;
	private JButton botCerrarTodo;

	public ListaDocumentos(ModPresentacionVentanaPrincipal modVentanaPrincipal)
			throws ExcepcionAnalisis, ExcepcionTransformacion {
		super();

		this.tema = new ShapedGradientTheme();
		this.titledTabProperties = new TitledTabProperties();
		this.modVentanaPrincipal = modVentanaPrincipal;
		this.docVacio            = new ModPresentacionDocumento(modVentanaPrincipal);

		// Botón de Cerrar Todo
		botCerrarTodo = createCloseAllTabsButton(this);
		setTabAreaComponents(new JComponent[] { botCerrarTodo });

		TabbedPanelProperties tpProps = getProperties();

		// Destacado al poner el cursor sobre un elemento
		tpProps.setHoverPolicy(TabbedPanelHoverPolicy.ALWAYS);

		// Reordenado de elementos posible
		tpProps.setTabReorderEnabled(true);

		// Aplicamos el tema
		tpProps.addSuperObject(tema.getTabbedPanelProperties());
		titledTabProperties.addSuperObject(tema.getTitledTabProperties());

		// Selección directa de una página
		tpProps
				.setTabDropDownListVisiblePolicy(TabDropDownListVisiblePolicy.MORE_THAN_ONE_TAB);
		tpProps.setEnsureSelectedTabVisible(true);
	}

	/* -- METODOS PUBLICOS -- */

	/**
	 * Añade una nueva pestaña con el título <code>s</code> y una vista de documento.
	 * 
	 * @param modP
	 *            Modelo de presentación del documento.
	 */
	public void agregarTab(File f)
			throws ExcepcionAnalisis {
		TabDocumento tab = new TabDocumento(f, modVentanaPrincipal);
		tab.setTitleComponent(createCloseTabButton(tab));
		tab.getProperties().addSuperObject(titledTabProperties);
		addTab(tab);
	}

	/**
	 * Selecciona la pestaña con el título <code>titSeleccion</code>.
	 * 
	 * @return <code>true</code> si se ha seleccionado tal pestaña,
	 *         <code>false</code> de lo contrario.
	 */
	public boolean seleccionarTab(File f) {
		final Tab nuevaSel = buscarTab(f);
		if (nuevaSel != null) {
			setSelectedTab(nuevaSel);
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Devuelve la vista del documento actual.
	 */
	public VistaDocumento getVistaTabActual() {
		return ((TabDocumento)getSelectedTab()).getVista();
	}

	/**
	 * Devuelve el documento actual.
	 */
	public ModPresentacionDocumento getDocumentoActual() {
		TabDocumento tabActual = (TabDocumento)getSelectedTab();
		if (tabActual != null) 
			return tabActual.getModPresentacionDocumento();
		return docVacio;
	}

	/**
	 * Cierra la pestaña actualmente seleccionada.
	 */
	public void cerrarTabActual() {
		Tab selTab = getSelectedTab();

		if (selTab != null)
			removeTab(selTab);
	}

	/**
	 * Cierra todas las pestañas.
	 */
	public void cerrarTodo() {
		// Iterate over all tabs and remove them.
		int tabCount = getTabCount();
		for (int i = 0; i < tabCount; i++)
			removeTab(getTabAt(0));
	}

	/**
	 * Sincroniza la lista con los modelos de presentación almacenados.
	 */
	public void sincronizar() throws ExcepcionAnalisis {
		TabDocumento tabActual = (TabDocumento)getSelectedTab();
		File fichActual        = tabActual.getFichero();
		cerrarTodo();		
		
		for (File f : modVentanaPrincipal.getDocumentosAbiertos()) {
			agregarTab(f);
		}
		seleccionarTab(fichActual);
		((TabDocumento)getSelectedTab()).sincronizar();		
	}

	/* -- METODOS PRIVADOS: código tomado de los ejemplos de InfoNode -- */

	/**
	 * Busca una pestaña por su título.
	 * 
	 * @param titulo
	 *            Título de la pestaña a buscar.
	 * @return La pestaña buscada si existe, o <code>null</code> si no.
	 */
	private Tab buscarTab(File f) {
		for (int i = 0; i < getTabCount(); ++i) {
			final TabDocumento tab = (TabDocumento) getTabAt(i);

			if (tab.getFichero().equals(f)) {
				return tab;
			}
		}

		return null;
	}

	/**
	 * Creates a close tab button that closes the given tab when the button is
	 * selected
	 * 
	 * @param tab
	 *            the tab what will be closed when the button is pressed
	 * @return the close button
	 */
	private JButton createCloseTabButton(final TitledTab tab) {
		final JButton closeButton = createXButton();
		closeButton.setToolTipText(res.getString("action_close_d"));

		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Closing the tab by removing it from the tabbed panel it is a
				// member of
				tab.getTabbedPanel().removeTab(tab);
			}
		});
		return closeButton;
	}

	/**
	 * Creates a close all tabs button that closes all tabs in the given Tabbed
	 * Panel when the button is selected
	 * 
	 * @param tabbedPanel
	 *            the tabbed panel that the button will close all tabs in
	 * @return the close button
	 */
	private JButton createCloseAllTabsButton(final TabbedPanel tabbedPanel) {
		final JButton closeButton = createXButton();
		closeButton.setToolTipText(res.getString("action_closeall_d"));

		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cerrarTodo();
			}
		});

		// Add a tab listener to the tabbed panel so that we know when tabs are
		// added
		// and removed so that the close button can be hidden when the tabbed
		// panel
		// doesn't contain any tabs.
		tabbedPanel.addTabListener(new TabAdapter() {
			public void tabAdded(TabEvent event) {
				closeButton.setVisible(true);
			}

			public void tabRemoved(TabRemovedEvent event) {
				// Hide the close button when there are no tabs in the tabbed
				// panel
				closeButton
						.setVisible(event.getTabbedPanel().getTabCount() > 0);
			}
		});
		return closeButton;
	}

	/**
	 * Creates a JButton with an X
	 * 
	 * @return the button
	 */
	private JButton createXButton() {
		final JButton closeButton = new JButton(iconoDesel);
		closeButton.setOpaque(false);
		closeButton.setMargin(null);
		closeButton.setBorder(new EmptyBorder(1, 1, 1, 1));

		closeButton.addMouseListener(new MouseAdapter() {
			public void mouseExited(MouseEvent e) {
				closeButton.setIcon(iconoDesel);
			}

			public void mouseEntered(MouseEvent e) {
				closeButton.setIcon(iconoSel);
			}
		});

		return closeButton;
	}
	
	/**** REACCIÓN ANTE EVENTOS DE TABS *****/

	public void tabAdded(TabEvent arg0) {
		// nada por defecto
	}

	public void tabDehighlighted(TabStateChangedEvent arg0) {
		// nada por defecto		
	}

	public void tabDeselected(TabStateChangedEvent arg0) {
		// nada por defecto		
	}

	public void tabDragAborted(TabEvent arg0) {
		// nada por defecto
	}

	public void tabDragged(TabDragEvent arg0) {
		// nada por defecto
	}

	public void tabDropped(TabDragEvent arg0) {
		// nada por defecto
	}

	public void tabHighlighted(TabStateChangedEvent arg0) {
		// nada por defecto
	}

	public void tabMoved(TabEvent arg0) {
		// nada por defecto
	}

	public void tabRemoved(TabRemovedEvent arg0) {
		modVentanaPrincipal.cierreDocumento(((TabDocumento)arg0.getTab()).getFichero());		
	}

	public void tabSelected(TabStateChangedEvent arg0) {
		TabDocumento tabActual = (TabDocumento)arg0.getTab();
		if (tabActual == null) return;
		
		File fichActual = tabActual.getFichero();
		ModPresentacionDocumento mp = modVentanaPrincipal.getModeloDocumento(fichActual);
		VistaDocumento vDoc = (VistaDocumento)tabActual.getContentComponent();
		
		vDoc.sincronizar(mp);		
	}
}
