package es.uca.xmleye.gui;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;
import javax.swing.*;
import javax.swing.Timer;
import javax.swing.tree.TreePath;
import javax.swing.border.EmptyBorder;
import javax.swing.event.*;
import java.util.concurrent.Callable;
import org.w3c.dom.*;

import net.infonode.tabbedpanel.TabAdapter;
import net.infonode.tabbedpanel.TabRemovedEvent;
import net.infonode.tabbedpanel.TabStateChangedEvent;

import es.uca.xmleye.Aplicacion;
import es.uca.xmleye.ExcepcionAnalisis;
import es.uca.xmleye.eventos.EventoBusqueda;
import es.uca.xmleye.eventos.EventoCambioFichero;
import es.uca.xmleye.eventos.ObservadorBusqueda;
import es.uca.xmleye.eventos.ObservadorCambioFichero;
import es.uca.xmleye.gui.util.AccionResource;
import es.uca.xmleye.gui.util.BotonResource;
import es.uca.xmleye.gui.util.FiltroExtensiones;
import es.uca.xmleye.gui.util.MenuOpcion;
import es.uca.xmleye.gui.util.MenuResource;
import es.uca.xmleye.presentacion.ModPresentacionDocumento;
import es.uca.xmleye.presentacion.ModPresentacionVentanaPrincipal;
import es.uca.xmleye.presentacion.ModPresentacionVentanaTipos;
import es.uca.xmleye.tipos.ExcepcionTipo;
import es.uca.xmleye.util.FunctorUnario;
import es.uca.xmleye.util.Par;
import es.uca.xmleye.util.ServidorAsincronoEjecucion;
import es.uca.xmleye.xslt.ExcepcionTransformacion;

public class VentanaPrincipal extends JFrame implements TreeSelectionListener,
		ObservadorBusqueda, ObservadorCambioFichero, HyperlinkListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 3584670885171884520L;

	// Intervalo de sincronización de estado
	private final static int INTERVALO_SINC_ESTADO = 250;

	// Aplicación principal
	private final Aplicacion aplicacion;

	// Modelo de presentación
	private final ModPresentacionVentanaPrincipal modPresentacion;
	// Modelo de presentación de tipos
	private final ModPresentacionVentanaTipos modPresentacionTipos;

	// Servicio de ejecución de tareas
	private final ServidorAsincronoEjecucion servEjecucion;

	// Recursos para internacionalización
	private static final ResourceBundle res;
	private static final ResourceBundle errRes;
	private static final ResourceBundle miscRes;

	// Acciones
	private AccionResource accionActivarAutoImport;
	private AccionResource accionAcercaDe;
	private AccionResource accionPrimerHijo;
	private AccionResource accionContraerTodo;
	private AccionResource accionEditar;
	private AccionResource accionSalir;
	private AccionResource accionExpandirTodo;
	private AccionResource accionBuscar;
	private AccionResource accionBuscarSig;
	private AccionResource accionHermanoSig;
	private AccionResource accionAbrir;
	private AccionResource accionPadre;
	private AccionResource accionHermanoAnt;
	private AccionResource accionSelHojaPreproc;
	private AccionResource accionSelHojaVista;
	private AccionResource accionEditarTipos;

	// / Tamaño mínimo
	final static int ANCHO_MIN = 640;
	final static int ALTO_MIN = 480;

	// Subventanas
	private VentanaBusqueda vntBusqueda;

	// Elementos a sincronizar de vez en cuando
	private JMenu menuFichero;
	private JMenu menuPreferencias;
	private MenuOpcion menuHojaPreprocesado;
	private MenuOpcion menuHojaVisualizacion;
	private JCheckBoxMenuItem chkHabilitarAutoImportar;
	private JLabel estadoPrograma;
	private JProgressBar progresoAccion;
	private ListaDocumentos listaDocs;

	static {
		res = ResourceBundle.getBundle(
				"es.uca.xmleye.gui.l10n.ResVentanaPrincipal", Locale
						.getDefault());
		errRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.ErrorMessages",
				Locale.getDefault());
		miscRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.MiscMessages",
				Locale.getDefault());
	}

	/**
	 * Crea la ventana principal de la aplicación y la muestra.
	 * 
	 * @param titulo
	 *            Título de la ventana.
	 * @param aplicacion
	 *            Moderador de la aplicación.
	 * @param docsPorAbrir
	 *            Documentos a abrir (pasados por línea de órdenes).
	 */
	public VentanaPrincipal(String titulo, Aplicacion aplicacion,
			List<File> fichPorAbrir) throws ExcepcionAnalisis, ExcepcionTipo,
			ExcepcionTransformacion {
		super(titulo);

		this.aplicacion = aplicacion;
		this.servEjecucion = new ServidorAsincronoEjecucion();
		this.modPresentacionTipos = new ModPresentacionVentanaTipos();
		this.modPresentacion = new ModPresentacionVentanaPrincipal(
				modPresentacionTipos);

		inicPropiedades();
		inicComponentes();
		inicAcciones();
		inicBarraMenu();
		inicBarraHerramientas();
		inicBarraEstado();

		pack();
		sincronizarDocumento();
		sincronizarEstado();

		setVisible(true);

		// Hacemos que abra todos los documentos pasados por la línea de órdenes
		for (File fich : fichPorAbrir) {
			abrirFichero(fich, null);
		}

	}

	public void valueChanged(TreeSelectionEvent e) {
		final ModPresentacionDocumento docActual = listaDocs
				.getDocumentoActual();
		final TreePath nuevaSel = e.getNewLeadSelectionPath();

		servEjecucion.ejecutar(new Callable<String>() {
			public String call() throws Exception {
				return docActual.solicitudCambioSeleccion(nuevaSel);
			}
		}, new FunctorUnario<String>() {
			public void ejecutar(String nuevoXHTML) {
				docActual.finCambioSeleccion(nuevoXHTML, nuevaSel);
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						sincronizarDocumento();
					}
				});
			}
		}, new FunctorUnario<Exception>() {
			public void ejecutar(Exception ex) {
				ex.printStackTrace();
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						sincronizarDocumento();
					}
				});
			}
		});
	}

	public void busquedaLanzada(final EventoBusqueda ev) {
		final ModPresentacionDocumento docActual = listaDocs
				.getDocumentoActual();

		servEjecucion.ejecutar(new Callable<NodeList>() {
			public NodeList call() throws Exception {
				return docActual.solicitudBusqueda(ev);
			}
		}, new FunctorUnario<NodeList>() {
			public void ejecutar(final NodeList res) {
				docActual.finBusqueda(res, ev);
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (res.getLength() == 0) {
							GestorMensajesSingleton
									.getInstancia()
									.mostrarError(
											miscRes.getString("find_nomatches"));
						}
						sincronizarDocumento();
					}
				});
			}
		}, new FunctorUnario<Exception>() {
			public void ejecutar(Exception ex) {
				ex.printStackTrace();
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
			}
		});
	}

	public void ficheroCambiado(EventoCambioFichero e) {
		abrirFichero(e.getFicheroCambiado(), null);
	}

	public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType() != HyperlinkEvent.EventType.ACTIVATED)
			return;

		final String link = e.getDescription();
		final Matcher m = Pattern.compile("([^#]+)(?:#xpointer\\((.+)\\))?")
				.matcher(link);

		if (!m.matches())
			return;

		final String ruta = m.group(1);
		final String xpointer = m.group(2);
		File f = new File(ruta);

		if (f.canRead()) {
			// Ruta relativa respecto a actual / absoluta:
			// no hay que hacer nada más
			abrirFichero(f, xpointer);
		} 
		else {

			// No sirve la dirección tal y como está: usaremos el directorio
			// base del documento actual.
			ModPresentacionDocumento docActual = listaDocs.getDocumentoActual();
			final String dirBase = docActual.getRutaBase();
			f = new File(dirBase + File.separator + ruta);
			
			abrirFichero(f, xpointer);
		}
	}

	/* --- METODOS PROTEGIDOS -------------------------------- */

	/**
	 * Inicializa las propiedades del diálogo.
	 */
	protected void inicPropiedades() {
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		setContentPane(panel);

		/** Avisaremos al controlador de cuando el usuario cierre esta ventana */
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent ev) {
				aplicacion.cerrar();
			}
		});

		/** Aseguramos un tamaño mínimo de la ventana */

		// Esto sólo funciona a partir de Mustang (J2SE 6) como debería ser
		setPreferredSize(new Dimension(ANCHO_MIN, ALTO_MIN));

		/*
		 * Usamos esta solución a un bug que ha permanecido desde J2SE 2
		 * (publicado en 09/03/2000, resuelto en Mustang (!) ). No es ideal,
		 * pero es la mejor que hay.
		 * 
		 * (Ver http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4320050)
		 */
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				int w = getWidth();
				int h = getHeight();
				if (w < ANCHO_MIN || h < ALTO_MIN) {
					pack();
				}
			}
		});
	}

	protected void inicComponentes() throws ExcepcionAnalisis,
			ExcepcionTransformacion {
		// LISTA DE DOCUMENTOS
		listaDocs = new ListaDocumentos(modPresentacion);
		listaDocs.setPreferredSize(new Dimension(ANCHO_MIN, 350));
		listaDocs.addTabListener(listaDocs);
		getContentPane().add(listaDocs);
	}

	protected void inicAcciones() {
		/* Las acciones simples se crean aquí como clases anónimas. */
		accionAcercaDe = new AccionResource(res, "action_about") {
			/**
			 * 
			 */
			private static final long serialVersionUID = 1791195432791531091L;

			public void actionPerformed(ActionEvent e) {
				new VentanaAcercaDe(VentanaPrincipal.this);
			}
		};
		accionPrimerHijo = new AccionResource(res, "action_child", res
				.getString("action_child_i")) {
			/**
			 * 
			 */
			private static final long serialVersionUID = -9011637049355624750L;

			public void actionPerformed(ActionEvent e) {
				listaDocs.getVistaTabActual().irPrimerHijo();
			}
		};
		accionContraerTodo = new AccionResource(res, "action_collapse") {
			/**
			 * 
			 */
			private static final long serialVersionUID = -3673103902091478796L;

			public void actionPerformed(ActionEvent e) {
				listaDocs.getVistaTabActual().contraerTodo();
			}
		};
		accionEditar = new AccionResource(res, "action_edit", res
				.getString("action_edit_i")) {
			/**
			 * 
			 */
			private static final long serialVersionUID = -8958694305808734851L;

			public void actionPerformed(ActionEvent e) {
				try {
					listaDocs.getDocumentoActual().editarDocumento();
				} catch (Exception ex) {
					ex.printStackTrace();
					GestorMensajesSingleton.getInstancia().mostrarError(
							ex.getMessage());
				}
			}
		};
		accionExpandirTodo = new AccionResource(res, "action_expand") {
			/**
			 * 
			 */
			private static final long serialVersionUID = 1461654585040886733L;

			public void actionPerformed(ActionEvent e) {
				listaDocs.getVistaTabActual().expandirTodo();
			}
		};
		accionHermanoSig = new AccionResource(res, "action_nextsibling", res
				.getString("action_nextsibling_i")) {
			/**
			 * 
			 */
			private static final long serialVersionUID = 5855016252380449128L;

			public void actionPerformed(ActionEvent e) {
				listaDocs.getVistaTabActual().irSigHermano();
			}
		};
		accionPadre = new AccionResource(res, "action_parent", res
				.getString("action_parent_i")) {
			/**
			 * 
			 */
			private static final long serialVersionUID = 8506197212134279952L;

			public void actionPerformed(ActionEvent e) {
				listaDocs.getVistaTabActual().irPadre();
			}
		};
		accionHermanoAnt = new AccionResource(res, "action_prevsibling", res
				.getString("action_prevsibling_i")) {
			/**
			 * 
			 */
			private static final long serialVersionUID = -4688738546929618943L;

			public void actionPerformed(ActionEvent e) {
				listaDocs.getVistaTabActual().irAntHermano();
			}
		};

		/**
		 * Acción de (des)activación de importación automática.
		 */
		accionActivarAutoImport = new AccionResource(res,
				"action_toggleautoimport") {
			/**
			 * 
			 */
			private static final long serialVersionUID = 3595025733061926016L;

			public void actionPerformed(ActionEvent e) {
				modPresentacion.cambioAutoImportacion();
				sincronizarDocumento();
			}
		};

		accionSalir = new AccionSalir();
		accionBuscar = new AccionBuscar();
		accionBuscarSig = new AccionBuscarSiguiente();
		accionAbrir = new AccionAbrir();

		accionSelHojaPreproc = new AccionCambioHojaPreprocesado();
		accionSelHojaVista = new AccionCambioHojaVista();

		accionEditarTipos = new AccionEdicionTipos();
	}

	/**
	 * Inicializa la barra de menús.
	 */
	protected void inicBarraMenu() {
		JMenuBar menu = new JMenuBar();
		menu.setPreferredSize(new Dimension(350, 20));

		// INICIO MENÚ FICHERO
		{
			menuFichero = new MenuResource(res, "menu_file");
			sincronizarMenuFichero(menuFichero);
			menu.add(menuFichero);
		}

		// INICIO MENÚ NAVEGAR
		{
			JMenu menuNavegar = new MenuResource(res, "menu_browse");
			menuNavegar.add(new JMenuItem(accionBuscar));
			menuNavegar.add(new JMenuItem(accionBuscarSig));
			menuNavegar.addSeparator();
			menuNavegar.add(new JMenuItem(accionPadre));
			menuNavegar.add(new JMenuItem(accionPrimerHijo));
			menuNavegar.add(new JMenuItem(accionHermanoAnt));
			menuNavegar.add(new JMenuItem(accionHermanoSig));
			menuNavegar.addSeparator();
			menuNavegar.add(new JMenuItem(accionExpandirTodo));
			menuNavegar.add(new JMenuItem(accionContraerTodo));
			menu.add(menuNavegar);
		}

		// INICIO MENÚ PREFERENCIAS
		{
			menuPreferencias = new MenuResource(res, "menu_preferences");
			sincronizarMenuPreferencias(menuPreferencias);
			menu.add(menuPreferencias);
		}

		// INICIO MENÚ AYUDA
		{
			JMenu menuHelp = new MenuResource(res, "menu_help");
			menuHelp.add(new JMenuItem(accionAcercaDe));
			menu.add(Box.createHorizontalGlue());
			menu.add(menuHelp);
		}

		setJMenuBar(menu);
	}

	/**
	 * Inicializa la barra de herramientas.
	 */
	protected void inicBarraHerramientas() {
		JToolBar barraHerramientas = new JToolBar(res
				.getString("toolbar_main_title"));
		barraHerramientas.setBorder(new EmptyBorder(1, 1, 1, 1));

		barraHerramientas
				.add(new BotonResource(res, accionAbrir, "action_open"));
		barraHerramientas.add(new BotonResource(res, accionEditar,
				"action_edit"));
		barraHerramientas.addSeparator();
		barraHerramientas.add(new BotonResource(res, accionBuscar,
				"action_find"));
		barraHerramientas.add(new BotonResource(res, accionPadre,
				"action_parent"));
		barraHerramientas.add(new BotonResource(res, accionPrimerHijo,
				"action_child"));
		barraHerramientas.add(new BotonResource(res, accionHermanoAnt,
				"action_prevsibling"));
		barraHerramientas.add(new BotonResource(res, accionHermanoSig,
				"action_nextsibling"));
		barraHerramientas.addSeparator();
		barraHerramientas
				.add(new BotonResource(res, accionSalir, "action_exit"));
		barraHerramientas.add(Box.createHorizontalGlue());

		getContentPane().add(barraHerramientas, 0);
	}

	/**
	 * Inicializa la barra de estado.
	 */
	protected void inicBarraEstado() {
		final JPanel panelBarraEstado = new JPanel(new BorderLayout());
		panelBarraEstado.setBorder(BorderFactory.createEmptyBorder(3, 6, 3, 3));
		panelBarraEstado.setMaximumSize(new Dimension(1500, 40));

		// Mensaje de estado
		estadoPrograma = new JLabel();
		estadoPrograma.setPreferredSize(new Dimension(300, 20));
		panelBarraEstado.add(estadoPrograma, BorderLayout.WEST);

		// Progreso de acción
		progresoAccion = new JProgressBar(JProgressBar.HORIZONTAL, 0, 100);
		progresoAccion.setIndeterminate(false);
		progresoAccion.setValue(100);
		panelBarraEstado.add(progresoAccion, BorderLayout.EAST);

		// Comprobación en cada medio segundo
		new Timer(INTERVALO_SINC_ESTADO, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sincronizarEstado();
			}
		}).start();

		getContentPane().add(panelBarraEstado);
	}

	/* --------------- METODOS PRIVADOS ------------------- */

	/* -- SINCRONIZACION: NIVEL DE APLICACION -- */

	/**
	 * (Re)construye el menú archivo, suponiendo que las acciones ya están
	 * creadas.
	 * 
	 * @param menuFichero
	 *            Menú al que modificar sus elementos.
	 */
	private void sincronizarMenuFichero(JMenu menuFichero) {
		menuFichero.removeAll();

		menuFichero.add(new JMenuItem(accionAbrir));
		menuFichero.add(new JMenuItem(accionEditar));

		// Documentos recientes
		List<String> hist = modPresentacion.getEntradasHistorial();
		if (hist.size() > 0) {
			menuFichero.addSeparator();

			for (ListIterator<String> it = hist.listIterator(); it.hasNext();) {
				final String elem = it.next();

				JMenuItem elemHistorial = new JMenuItem("... "
						+ elem.substring(Math.max(0, elem.length() - 20), elem
								.length()));
				elemHistorial.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						abrirFichero(new File(elem), null);
					}
				});

				menuFichero.add(elemHistorial);
			}
		}

		menuFichero.addSeparator();
		menuFichero.add(new JMenuItem(accionSalir));
	}

	/**
	 * Añade los elementos necesarios al menú Fichero. Útil para actualizar sus
	 * contenidos al cambiar el modelo.
	 * 
	 * @param menuPrefs
	 *            Menú al que modificar sus elementos.
	 */
	private void sincronizarMenuPreferencias(JMenu menuPrefs) {
		menuPrefs.removeAll();

		menuHojaPreprocesado = new MenuOpcion(accionSelHojaPreproc,
				modPresentacion.getNombresHojasPreproc(), listaDocs
						.getDocumentoActual().getNombreHojaPreproc());
		menuHojaVisualizacion = new MenuOpcion(accionSelHojaVista,
				modPresentacion.getNombresHojasVista(), listaDocs
						.getDocumentoActual().getNombreHojaVista());
		menuPrefs.add(menuHojaPreprocesado);
		menuPrefs.add(menuHojaVisualizacion);
		menuPrefs.addSeparator();

		chkHabilitarAutoImportar = new JCheckBoxMenuItem(
				accionActivarAutoImport);
		sincronizarAutoImportar();

		menuPrefs.add(chkHabilitarAutoImportar);
		menuPrefs.addSeparator();

		menuPrefs.add(new JMenuItem(accionEditarTipos));

		// AÑADIR DIÁLOGO FILTROS
	}

	/* -- SINCRONIZACION: MEDIO NIVEL -- */

	private void sincronizarEstado() {
		if (modPresentacion.getOcupado()) {
			progresoAccion.setValue(0);
			progresoAccion.setIndeterminate(true);
		} else {
			progresoAccion.setValue(100);
			progresoAccion.setIndeterminate(false);
		}

		estadoPrograma.setText(modPresentacion.getMensajeEstado());
	}

	private void sincronizarDocumento() {
		final ModPresentacionDocumento modDocActual = listaDocs
				.getDocumentoActual();

		// Marcamos las hojas usadas en ambos menús
		menuHojaPreprocesado.setSeleccion(modDocActual.getNombreHojaPreproc());
		menuHojaVisualizacion.setSeleccion(modDocActual.getNombreHojaVista());

		setTitle(modPresentacion.getTitulo(listaDocs.getDocumentoActual()));

		sincronizarAutoImportar();
		habilitarManipulacion(modDocActual.getNavegable());
		if (listaDocs.getSelectedTab() != null) {
			((TabDocumento) listaDocs.getSelectedTab()).sincronizar();
		}
	}

	/* -- SINCRONIZACION: BAJO NIVEL -- */

	/**
	 * Sincroniza sólo la marca de autoimportación.
	 */
	private void sincronizarAutoImportar() {
		chkHabilitarAutoImportar.setSelected(modPresentacion
				.getActualizacionAutomatica());
	}

	/**
	 * Activa o desactiva todas las acciones que permiten operar sobre el
	 * documento actual.
	 * 
	 * @param b
	 *            Indica si se activan (<code>true</code>) o desactivan (<code>false</code>)
	 *            los controles.
	 */
	private void habilitarManipulacion(boolean b) {
		accionPrimerHijo.setEnabled(b);
		accionContraerTodo.setEnabled(b);
		accionEditar.setEnabled(b);
		accionExpandirTodo.setEnabled(b);
		accionBuscar.setEnabled(b);
		accionBuscarSig.setEnabled(b);
		accionHermanoSig.setEnabled(b);
		accionPadre.setEnabled(b);
		accionHermanoAnt.setEnabled(b);
		accionSelHojaPreproc.setEnabled(b);
		accionSelHojaVista.setEnabled(b);
	}

	/** --- OTRAS OPERACIONES --- */

	/**
	 * Abre el documento con fichero <code>fuente</code>, posiblemente
	 * haciendo una búsqueda por la ruta <code>xpointer</code> tras ello.
	 * 
	 * @param fuente
	 *            Fichero a abrir.
	 * @param xpath
	 *            Ruta XPath a seleccionar (opcional).
	 */
	private void abrirFichero(final File fuente, final String xpath) {
		servEjecucion.ejecutar(new Callable<ModPresentacionDocumento>() {
			public ModPresentacionDocumento call() throws Exception {
				return modPresentacion.solicitudAbrirDocumento(fuente);
			}
		}, new FunctorUnario<ModPresentacionDocumento>() {
			public void ejecutar(final ModPresentacionDocumento nDoc) {
				try {					
					final boolean nuevoDocumento = modPresentacion
							.agregarDocumento(nDoc);
					nDoc.getNotCambios().agregarObservadorCambioFichero(
							(ObservadorCambioFichero) VentanaPrincipal.this);

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							try {
								sincronizarMenuFichero(menuFichero);
								if (nuevoDocumento) {
									listaDocs.agregarTab(fuente);
									listaDocs.seleccionarTab(fuente);

									final VistaDocumento vista = listaDocs
											.getVistaTabActual();
									vista
											.addTreeSelectionListener((TreeSelectionListener) VentanaPrincipal.this);
									vista
											.addHyperlinkListener((HyperlinkListener) VentanaPrincipal.this);
								} else {
									listaDocs.seleccionarTab(fuente);
								}
								sincronizarDocumento();

								if (xpath != null) {
									EventoBusqueda ev = new EventoBusqueda();
									ev.setRutaXPath(xpath);
									busquedaLanzada(ev);
								}
							} catch (Exception ex) {
								ex.printStackTrace();
							}
						}
					});
				} catch (ExcepcionTransformacion ex) {
					ex.printStackTrace();
				}
			}
		}, new FunctorUnario<Exception>() {
			public void ejecutar(Exception ex) {
				ex.printStackTrace();
				GestorMensajesSingleton.getInstancia().mostrarError(
						errRes.getString("err_open_fail") + "\n"
								+ ex.getLocalizedMessage());
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						sincronizarMenuFichero(menuFichero);
					}
				});
			}
		});

	}

	/* --- CLASES PRIVADAS ------------------------------ */

	/* ACCIONES - sólo las más complejas */

	/** Acción de apertura de documentos. */
	private class AccionAbrir extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = -3033019833259042023L;

		public AccionAbrir() {
			super(res, "action_open", res.getString("action_open_i"));
		}

		public void actionPerformed(ActionEvent e) {
			// Creamos un selector de ficheros actualizado con los
			// tipos admitidos actualmente
			JFileChooser dlgAbrir = new JFileChooser();
			dlgAbrir.setMultiSelectionEnabled(false);

			final List<Par<String, String[]>> tipos = modPresentacionTipos
					.getElemsSelector();
			for (Par<String, String[]> entrada : tipos) {
				dlgAbrir.addChoosableFileFilter(new FiltroExtensiones(entrada
						.segundo(), entrada.primero()));
			}

			if (dlgAbrir.showOpenDialog(VentanaPrincipal.this) == JFileChooser.APPROVE_OPTION) {
				abrirFichero(dlgAbrir.getSelectedFile(), null);
			}
		}
	}

	/** Acción de lanzamiento de diálogo Buscar. */
	private class AccionBuscar extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = -5295820489091250276L;

		public AccionBuscar() {
			super(res, "action_find", res.getString("action_find_i"));
		}

		public void actionPerformed(ActionEvent e) {
			if (vntBusqueda == null) {
				vntBusqueda = new VentanaBusqueda(VentanaPrincipal.this);
				vntBusqueda
						.agregarObservadorBusqueda((ObservadorBusqueda) VentanaPrincipal.this);
			}

			vntBusqueda.setVisible(true);
		}
	}

	/** Acción de búsqueda de la siguiente coincidencia. */
	private class AccionBuscarSiguiente extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = 6994490721852391963L;

		public AccionBuscarSiguiente() {
			super(res, "action_findnext");
		}

		public void actionPerformed(ActionEvent e) {
			final ModPresentacionDocumento docActual = listaDocs
					.getDocumentoActual();

			servEjecucion.ejecutar(new Callable<NodeList>() {
				public NodeList call() throws Exception {
					return docActual.repetirBusqueda();
				}
			}, new FunctorUnario<NodeList>() {
				public void ejecutar(final NodeList res) {
					docActual.finBusqueda(res, null);
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (res == null) {
								GestorMensajesSingleton
										.getInstancia()
										.mostrarError(
												errRes
														.getString("err_findnext_nofindyet"));
							} else if (res.getLength() == 0) {
								GestorMensajesSingleton
										.getInstancia()
										.mostrarError(
												miscRes
														.getString("find_nomatches"));
							}
							sincronizarDocumento();
						}
					});
				}
			}, new FunctorUnario<Exception>() {
				public void ejecutar(Exception ex) {
					GestorMensajesSingleton.getInstancia().mostrarError(
							ex.getMessage());
				}
			});

		}
	}

	/** Acción de salida del programa. */
	private class AccionSalir extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = -5183025387352671187L;

		public AccionSalir() {
			super(res, "action_exit", res.getString("action_exit_i"));
		}

		public void actionPerformed(ActionEvent e) {
			if (GestorMensajesSingleton.getInstancia().preguntarSiNo(
					res.getString("confirm_exit_msg"))) {
				aplicacion.cerrar();
			}
		}
	}

	/** Acción de cambio de hoja de preprocesado */
	private class AccionCambioHojaPreprocesado extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = -5434085806022651386L;

		public AccionCambioHojaPreprocesado() {
			super(res, "action_selectstylesheet_preproc");
		}

		public void actionPerformed(final ActionEvent e) {
			try {
				modPresentacion.cambioHojaPreprocesado(listaDocs
						.getDocumentoActual(), e.getActionCommand());
				sincronizarDocumento();
			} catch (ExcepcionTransformacion ex) {
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
				sincronizarDocumento();
			} catch (ExcepcionAnalisis ex) {
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
				sincronizarDocumento();
			}
		}
	}

	/** Acción de cambio de hoja de visualización */
	private class AccionCambioHojaVista extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = 361078597085383631L;

		public AccionCambioHojaVista() {
			super(res, "action_selectstylesheet_view");
		}

		public void actionPerformed(ActionEvent e) {
			try {
				modPresentacion.cambioHojaVisualizacion(listaDocs
						.getDocumentoActual(), e.getActionCommand());
				sincronizarDocumento();
			} catch (ExcepcionTransformacion ex) {
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
				sincronizarDocumento();
			}
		}
	}

	/** Acción de edición de tipos disponibles para el usuario */
	private class AccionEdicionTipos extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = -8226394759881245434L;

		public AccionEdicionTipos() {
			super(res, "action_edittypes");
		}

		public void actionPerformed(ActionEvent e) {
			new VentanaTipos(VentanaPrincipal.this, modPresentacionTipos);
		}
	}

}
