package es.uca.xmleye.gui.arboles;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import javax.swing.tree.TreeModel;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import java.util.Iterator;

/** 
    Iterador sobre un TreeModel basado en un recorrido en
    preorden. No permite quitar elementos.
*/

@SuppressWarnings("unchecked")
class IteradorPreorden implements Iterator {
  private TreeModel modelo;
  private LinkedList<Object> pila;
  
  /** 
      Crea un nuevo iterador para recorrer el TreeModel
      <code>m</code>.
      @param m TreeModel a recorrer en preorden.
  */
  public IteradorPreorden(TreeModel m) {
    this.modelo = m;
    this.pila = new LinkedList<Object>();
    pila.add(m.getRoot());
  }
  
  /** 
      Informa de si hay más elementos en la secuencia.
      @return <code>true</code> si quedan más elementos,
      <code>false</code> si no.
  */
  public boolean hasNext() {
    return !pila.isEmpty();
  }
  
  /** 
      Accede al siguiente elemento del TreeModel en preorden.
      @return Nodo a continuación en preorden.
      @throws NoSuchElementException No hay más elementos.
  */
  public Object next() throws NoSuchElementException {
    final Object res = pila.removeLast();
    
    int nHijos = modelo.getChildCount(res);
    for (int i=0; i < nHijos; ++i) {
      pila.add(modelo.getChild(res,i));
    }
    
    return res;
  }

  /** Método no implementado.
      @throws UnsupportedOperationException Método no implementado. */
  public void remove() throws UnsupportedOperationException, IllegalStateException {
    throw new UnsupportedOperationException();
  }

}
